c program to cull extracted SEABED data table for seafloor samples
c (within 0.5 meters) and average redundancies at same location
c In addition, utilize a gridded bathymetry map in ArcGIS ascii grid
c format to fill in missing depth values.

c ******************************************************************
c Program written by Dr John GOFF, University of Texas at Austin

c "http://www.ig.utexas.edu/people/staff/goff/"

c Work supported by ONR Awards, 2006-7
c ******************************************************************

      program cull

      parameter (nd=250000)
      parameter (ncmax=3000)
      parameter (nrmax=3000)

      real*8 lat(nd),lon(nd),latt,lont,latmin,lonmin,llint
      real gs(nd),sort(nd),top(nd),bottom(nd)
      real gst,sortt,topt,bottomt
      integer depth(nd),gravel(nd),sand(nd),mud(nd),clay(nd)
      integer deptht,gravelt,sandt,mudt,clayt
      integer bgrid(ncmax,nrmax),nodata
      character header*120, hline*30, c*2
      
c first read in bathymetric grid file.  Assuming here the ArcGIS 
c ascii raster format with integer meters (could easily be changed
c to real values). 
      c = ', ' 
      open (3,file='pac_bathy.asc',status='old')
      do i = 1,6
         read (3,'(a)') hline
         nb = 1
         do while (hline(nb:nb) .ne. ' ')
           nb = nb + 1 
         enddo
         nb = nb - 1
         if (hline(1:nb) .eq. 'NCOLS' .or. hline(1:nb) .eq. 'ncols') 
     #       read (hline(nb+2:30),*) ncols
         if (hline(1:nb) .eq. 'NROWS' .or. hline(1:nb) .eq. 'nrows') 
     #       read (hline(nb+2:30),*) nrows
         if (hline(1:nb) .eq. 'XLLCORNER' .or. hline(1:nb) .eq.  
     #       'xllcorner') read (hline(nb+2:30),*) lonmin
         if (hline(1:nb) .eq. 'YLLCORNER' .or. hline(1:nb) .eq.  
     #       'yllcorner') read (hline(nb+2:30),*) latmin
         if (hline(1:nb) .eq. 'CELLSIZE' .or. hline(1:nb) .eq.  
     #       'cellsize') read (hline(nb+2:30),*) llint     
         if (hline(1:nb) .eq. 'NODATA_VALUE' .or. hline(1:nb) .eq.  
     #       'nodata_value') read (hline(nb+2:30),*) nodata    
      enddo
      write (*,*) ncols,nrows,lonmin,latmin,llint,nodata
      do j = nrows,1,-1
         read (3,*) (bgrid(i,j),i=1,ncols)
      enddo
      
      close (3)
 
      open (1,file='pac_ext.data',status='old')
      open (2,file='pac_ext_sf.txt',status='unknown')

      write (2,'(a)') '"lat", "lon", "depth", '//
     #                ' "g", "s", "m", "c", "gs", "sort"'

c next go through the file and remove records below 0.5 m depth
c or of unknown mean grain size
      read (1,'(a)') header
      i = 0
10    read (1,*,end=90) latt,lont,deptht,topt,bottomt,gravelt,
     #           sandt,mudt,clayt,gst,sortt
c      write (*,*) latt,lont,deptht,topt,bottomt,gravelt,
c     #           sandt,mudt,clayt,gst,sortt
      if (topt .lt. 0.5 .and. bottomt .lt. 2.) then
         i = i + 1
         lat(i) = latt
         lon(i) = lont
         depth(i) = deptht
         top(i) = topt
         bottom(i) = bottomt
         gravel(i) = gravelt
         sand(i) = sandt
         mud(i) = mudt
         clay(i) = clayt
         gs(i) = gst
         sort(i) = sortt
      endif
      goto 10
90    close (1)
      numdata = i

100   format (f8.5,a2,f10.5,a2,i4,a2,i3,a2,i3,a2,i3,a2,i3,a2,
     #        f5.1,a2,f5.1)
c now average grain sizes at common locations, determine water
c depths of samples with "no data" depth fields, and write to file
      i = i + 1
      latt = lat(1) 
      lont = lon(1)
      deptht = depth(1)
      gravelt = gravel(1)
      sandt = sand(1)
      mudt = mud(1)
      clayt = clay(1)
      gst = gs(1)
      sortt = sort(1)
      xi = 1.
      xg = 1.
      xsa = 1.
      xm = 1.
      xc = 1.
      xso = 1.
      index = 0
      do i = 2,numdata
         if (lat(i) .eq. latt .and. lon(i) .eq. lont) then
            if (gravel(i) .ne. -99) then
               if (gravelt .eq. -99) then
                  gravelt = gravel(i)
               else
                  gravelt = gravelt + gravel(i)
                  xg = xg + 1.
               endif
            endif
            if (sand(i) .ne. -99) then
               if (sandt .eq. -99) then
                  sandt = sand(i)
               else
                  sandt = sandt + sand(i)
                  xsa = xsa + 1.
               endif
            endif
            if (mud(i) .ne. -99) then
               if (mudt .eq. -99) then
                  mudt = mud(i)
               else
                  mudt = mudt + mud(i)
                  xm = xm + 1.
               endif
            endif
            if (clay(i) .ne. -99) then
               if (clayt .eq. -99) then
                  clayt = clay(i)
               else
                  clayt = clayt + clay(i)
                  xc = xc + 1.
               endif
            endif
            if (sort(i) .ne. -99.) then
               if (sortt .eq. -99) then
                  sortt = sort(i)
               else
                  sortt = sortt + sort(i)
                  xso = xso + 1.
               endif
            endif
            gst = gst + gs(i)
            xi = xi + 1.
         else
            if (deptht .eq. -99) then
               ix = ifix((lont - lonmin)/llint) + 1
               xs = lonmin + (ix-1)*llint
               dx = (lont - xs)/llint
               iy = ifix((latt - latmin)/llint) + 1
               ys = latmin + (iy-1)*llint
               dy = (latt - ys)/llint
               if (ix .ge. 1 .and. ix .le. ncols .and. 
     #             iy .ge. 1 .and. iy .le. nrows) then 
                  if (bgrid(ix,iy) .ne. nodata .and. 
     #                bgrid(ix+1,iy) .ne. nodata .and.
     #                bgrid(ix,iy+1) .ne. nodata .and.
     #                bgrid(ix+1,iy+1) .ne. nodata) then
                     xdepth = (1.-dx)*(1.-dy)*bgrid(ix,iy) +
     #                        dx*(1.-dy)*bgrid(ix+1,iy) +
     #                        (1.-dx)*dy*bgrid(ix,iy+1) +
     #                         dx*dy*bgrid(ix+1,iy+1)
                     deptht = (-1)*ifix(xdepth - 0.5)
                  endif
               endif
            endif
            write (2,100) latt,c,lont,c,deptht,c,int(gravelt/xg),
     #            c,int(sandt/xsa),c,int(mudt/xm),c,int(clayt/xc),
     #            c,gst/xi,c,sortt/xso
            latt = lat(i) 
            lont = lon(i)
            deptht = depth(i)
            gravelt = gravel(i)
            sandt = sand(i)
            mudt = mud(i)
            clayt = clay(i)
            gst = gs(i)
            sortt = sort(i)
            xi = 1.
            xg = 1.
            xsa = 1.
            xm = 1.
            xc = 1.
            xso = 1.
         endif
         if (i .eq. numdata) then
            if (deptht .eq. -99) then
               ix = ifix((lont - lonmin)/llint) + 1
               xs = lonmin + (ix-1)*llint
               dx = (lont - xs)/llint
               iy = ifix((latt - latmin)/llint) + 1
               ys = latmin + (iy-1)*llint
               dy = (latt - ys)/llint
               if (ix .ge. 1 .and. ix .le. ncols .and. 
     #             iy .ge. 1 .and. iy .le. nrows) then 
                  if (bgrid(ix,iy) .ne. nodata .and. 
     #                bgrid(ix+1,iy) .ne. nodata .and.
     #                bgrid(ix,iy+1) .ne. nodata .and.
     #                bgrid(ix+1,iy+1) .ne. nodata) then
                     xdepth = (1.-dx)*(1.-dy)*bgrid(ix,iy) +
     #                        dx*(1.-dy)*bgrid(ix+1,iy) +
     #                        (1.-dx)*dy*bgrid(ix,iy+1) +
     #                         dx*dy*bgrid(ix+1,iy+1)
                     deptht = (-1)*ifix(xdepth - 0.5)
                  endif
               endif
            endif
            write (2,100) latt,c,lont,c,deptht,c,int(gravelt/xg),
     #            c,int(sandt/xsa),c,int(mudt/xm),c,int(clayt/xc),
     #            c,gst/xi,c,sortt/xso

         endif
      enddo

      stop
      end
 
            
       
