c program to compute a binned semivariogram from data file, and output
c by lat, semivariogram value, and number of data in each bin


c ******************************************************************
c Program written by Dr John GOFF, University of Texas at Austin

c "http://www.ig.utexas.edu/people/staff/goff/"

c Work supported by ONR Awards, 2006-7
c ******************************************************************

      program db_var

      parameter (nmax=50000)
      parameter (nbmax=500)

      character header*80,ifile*80,ofile*80,chd*1
      real*8 lat(nmax),lon(nmax),latmid,xdif,ydif,xscale
      real gs(nmax),b(nbmax),xcount(nbmax),dist,dif2,pi
      real x(nbmax),y(nbmax)
      integer ni(nbmax),depth(nmax)

      pi = 3.1415926
      write (*,*) 'enter maximum lag, km'
      read (*,*) xlmax

      write (*,*) 'enter number of bins (max = ',nbmax,')'
      read (*,*) nbin
      xbi = xlmax/float(nbin)

      do i = 1,nbmax
         b(i) = 0.
         xcount(i) = 0.
      enddo
      
      write (*,*) 'enter input file name'
      read (*,'(a)') ifile
      write (*,*) 'enter output file name'
      read (*,'(a)') ofile
      
      write (*,*) 'detrend grain size wrt water depth? (y or n)'
      read (*,'(a)') chd

      i = 1
      open (1,file=ifile,status='old')
      read (1,'(a)') header
10    read (1,*,end=90) lat(i),lon(i),depth(i),i2,i3,i4,i5,gs(i),x2
      i = i + 1
      goto 10
90    close (1)

      ndata = i - 1
      write (*,*) ndata
      
c GMT needed for the following detrending system call to work.      
      if (chd .eq. 'y') then
         open (4,file='dg.out',status='unknown')
         do i = 1,ndata
            write (4,*) depth(i),gs(i)
         enddo
         close (4)
         
         n = system('trend1d dg.out -Fxr -N2 >! dg_dt.out')
         
         open (5,file='dg_dt.out',status='old')
         do i = 1,ndata
            read (5,*) depth(i),gs(i)
         enddo
         close (5)
      endif

      sum = 0.
      do i = 1,ndata
         sum = sum + gs(i)
      enddo
      sum = sum/float(ndata)
      write (*,*) 'mean grain size: ',sum
      
      sum2 = 0.
      do i = 1,ndata
         sum2 = sum2 + (gs(i) - sum)**2
      enddo
      sum2 = sum2/float(ndata)
      write (*,*) 'grain size variance: ',sum2
      
      do i = 1,ndata-1
         do j = i+1,ndata
            latmid = (lat(j) + lat(i))/2.
            xscale = cos(latmid*pi/180.)
            ydif = 111.1111111*(lat(j) - lat(i))
            xdif = 111.1111111*(lon(j) - lon(i))*xscale
            dist = sqrt(xdif**2 + ydif**2)
            dif2 = 0.5*((gs(j)-gs(i))**2)
            ibin = 1 + ifix(dist/xbi)
            if (ibin .le. nbin) then
               b(ibin) = b(ibin) + dif2
               xcount(ibin) = xcount(ibin) + 1.
            endif
         enddo
      enddo

      nout = 0
      do i = 1,nbin
         if (xcount(i) .ge. 5.) then
            nout = nout + 1
            x(nout) = (i-0.5)*xbi
            y(nout) = b(i)/xcount(i)
            ni(nout) = ifix(xcount(i))
            if (i .eq. 1) write (*,*) '1st lag: ',
     #                  (i-0.5)*xbi,b(i)/xcount(i),xcount(i)
         endif
      enddo

      open (2,file=ofile,status='unknown')
      write (2,*) nout
      do i = 1,nout
         write (2,*) x(i),y(i),ni(i)
      enddo
      close (2)
 
      stop
      end