c this function calculates the modified bessel function (second kind) of order
c v (0<v<1) and length r


c ******************************************************************
c Program written by Dr John GOFF, University of Texas at Austin

c "http://www.ig.utexas.edu/people/staff/goff/"

c Work supported by ONR Awards, 2006-7
c ******************************************************************

      real function mbsl(v,r)
    
      integer gind1,gind2
      real v,r,gm(101),dif1,dif2,gam1,gam2,nfac,cpv,cmv,cip,cim
      real n,pi,mbsl1,mbsl2,rn
     
c gamma function from 1 to 2 at intervals of .01  
      data gm/1.00000,.99433,.98884,.98355,.97844,.97305,.96874,
     #         .96415,.95973,.95546,.95135,.94740,.94359,.93993,
     #         .93642,.93304,.92980,.92670,.92373,.92089,.91817,
     #         .91558,.91311,.91075,.90852,.90640,.90440,.90250,
     #         .90072,.89904,.89747,.89600,.89464,.89338,.89222,
     #         .89115,.89018,.88931,.88854,.88785,.88726,.88676,
     #         .88636,.88604,.88581,.88566,.88560,.88563,.88575,
     #         .88595,.88623,.88659,.88704,.88757,.88818,.88887,
     #         .88964,.89049,.89142,.89243,.89352,.89468,.89592,
     #         .89724,.89864,.90012,.90167,.90330,.90500,.90678,
     #         .90864,.91057,.91258,.91466,.91683,.91906,.92137,
     #         .92376,.92623,.92877,.93138,.93408,.93685,.93969,
     #         .94261,.94561,.94869,.95184,.95507,.95838,.96177,
     #         .96523,.96877,.97240,.97610,.97988,.98374,.98768,
     #         .99171,.99581,1.0000/                            
c woops guard       
      if (v .le. 0 .or. v .ge. 1) then
         write (*,*) 'I am not authorized to attempt that'
         mbsl = 0.
         goto 999
      elseif (r .le. 0.) then
         write (*,*) 'that is infinite - no way jose'
         mbsl = 0.
         goto 999
      endif 
      if (r .lt. 0.) then
         r = -r
      endif
          
      pi = 3.141592654
      rcut = 2.
      gind1 = ifix(100.*v) + 1
      dif1 = v - float(gind1-1)/100. 
c gamma function of 1+v
      gam1 = gm(gind1) + (dif1/.01)*(gm(gind1+1)-gm(gind1))
      gind2 = ifix(100.*(1.-v)) + 1
      dif2 = 1.-v - float(gind2-1)/100.
c gamma function of 1-v = gamma(2-v)/(1-v)
      gam2 = (gm(gind2) + (dif2/.01)*
     #       (gm(gind2+1)-gm(gind2)))/(1.-v)
c 0!
      nfac = 1. 
c 0th term in sum
      cpv = 1./gam1
      cmv = 1./gam2
      cip = 1.
      cim = 1. 
      n = 1. 
c summation
      do while (abs(cip) .gt. .0001 .and. abs(cim) .gt. .0001)
         gam1 = (n+v)*gam1
         gam2 = (n-v)*gam2
         nfac = n*nfac
         rn = (.5*r)**(2.*n)
         cip = rn/(nfac*gam1)
         cim = rn/(nfac*gam2)
         cpv = cpv + cip
         cmv = cmv + cim
         n = n + 1.
      enddo
      cpv = cpv*((.5*r)**v)
      cmv = cmv/((.5*r)**v)
      mbsl1 = pi*(cmv - cpv)/(2.*sin(v*pi)) 
      mbsl2 =  sqrt(pi*.5/r)*exp(-r)
     #       *(1 + (4.*v**2-1.)/(8.*r))
      if (v .gt. 0.5) then
         if (r .gt. 6) then
            mbsl = 0.
         elseif (r .lt. 6 .and. r .gt. rcut) then
            mbsl = mbsl2
         else
            mbsl = amin1(mbsl1,mbsl2)
         endif
      else
         if (r .gt. 6) then
            mbsl = 0.
         else
            mbsl = mbsl1
         endif
      endif

999   return
      end 

