      program plotsv 
      
c program to plot up to 5 semi-variogram and overlay von Karman models

c ******************************************************************
c Program written by Dr John GOFF, University of Texas at Austin

c "http://www.ig.utexas.edu/people/staff/goff/"

c Work supported by ONR Awards, 2006-7
c ******************************************************************


      parameter (nextra=1000)
      real rcor(5,nextra),xlag(5,nextra)
      real refh,refl,scalel,scaleh
      real max,nu(5),yos,mbsl,g0,k0(5),h2(5),tsz
      real offset(5),xintm
c      real poffset,ph2,pk0,pnu,goffset,h2g,kg
      real*8 xoff,yoff,xscl,yscl,pw,xx,yy,mx(nextra),my(nextra)
      real*8 x1,x2,y1,y2,angle,rx(nextra),ry(nextra),lx(2),ly(2)
      integer i,a1,a2,nlag(5)
      logical modelif,mpowerif,gmodif,rescaleif
      character xcp*10,ycp*10,choice*1
c      character*1 ch
      character*80 title,ifile(5),ofile,ttitle
          
      pi = 3.1415926
      write (*,*) 'input number of files?'
      read (*,*) nf
      do i = 1,nf
         write (*,*) 'input file name',i
         read (*,'(a)') ifile(i)
         open (1,file=ifile(i),status='old')
         read (1,*) nlag(i)
         do j = 1,nlag(i)
            read (1,*) xlag(i,j), rcor(i,j)
         enddo
         close (1)
      enddo

      h = 0.     
      write (*,*) 'intput title'
      read (*,'(a)') ttitle
      ttitle = ttitle(1:lenb(ttitle))//'\0'

c set some plot parameters
      data tall/7.0/,wide/7.0/ 
      bh = tall - 2.
      bw = wide - 2.
      refh = 1.5    
      refl = 1.5   
      modelif = .false. 
      mpowerif = .false.
      gmodif = .false.
      rescaleif = .false.
      yos = 2.
      tsz = .1
                 
      max = rcor(1,1)
      do k = 1,nf
         do i = 1,nlag(1)
            max = amax1(max,rcor(k,i))
         enddo
      enddo
      yls = max   
c nrs is the maximum vertical scale value, and nticks is the number
c of vertical ticks on the vertical axis
      write (*,*) 'suggested yls:',max
10    write (*,*) 'input yls, nticks'
      read (*,*) yls,nticks
      scaleh = bh/yls

      xlmax = amax1(xlag(1,nlag(1)),xlag(2,nlag(2)))
      write (*,*) 'suggested xls:',xlmax
      write (*,*) 'input xls, ntickl:'
      read (*,*) xls,ntickl
      scalel = bw/xls
      rescale = 1.

      xintm = xlmax/float(nextra) 
      mcorm = nextra

100   continue 
c open post-script file, draw border 
      xoff = 0.5
      yoff = 0.5
      pw = 8.5
      xscl = 1.
      yscl = 1.
      ofile = 'varplot.ps\0'
      call ps_plotinit (ofile,0,1,xoff,yoff,
     #                  xscl,yscl,1,300,1,pw)
      call ps_setfont (0)
      call ps_setline (5)
                     
      x1 = 0.
      y1 = 0.
      x2 = wide
      y2 = tall
c      call ps_rect (x1,y1,x2,y2,255,255,255,1)

c draw axes and tic marks
      lx(1) = refl
      ly(1) = refh
      lx(2) = refl
      ly(2) = bh + refh
      call ps_line(lx,ly,2,3,0)
      lx(2) = refl + bw
      ly(2) = refh
      call ps_line(lx,ly,2,3,0)
      do i=1,ntickl
         lx(1) = refl + i*(bw/float(ntickl))
         ly(1) = refh
         lx(2) = lx(1)
         ly(2) = refh + tsz
         call ps_line(lx,ly,2,3,0)
      enddo
      do i=1,nticks
         lx(1) = refl 
         ly(1) = refh + i*(bh/float(nticks))
         lx(2) = refl + tsz
         ly(2) = ly(1)
         call ps_line(lx,ly,2,3,0)
      enddo 

c draw labels                               
      xx = refl + bw/2.
      yy = refh - 0.5
      title = 'Lag, km\0'
      angle = 0.
      call ps_text (xx,yy,25,title,angle,2,0)
      title = 'Semi-Variogram, phi@+2@+\0'
      xx = refl - 0.5
      yy = refh + bh/2.
      angle = 270.
      call ps_text (xx,yy,25,title,angle,2,0) 
      xx = refl
      yy = tall + 0.2
      angle = 0.
      call ps_text (xx,yy,25,ttitle,angle,2,0)

      if (xls .lt. 0.1) then
         write (xcp,'(f4.2)') xls
      elseif (xls .lt. 1. .and. xls .ge. 0.1) then
         write (xcp,'(f3.1)') xls
      elseif (xls .lt. 100) then
         write (xcp,'(i2)') ifix(xls)
      else
         write (xcp,'(i3)') ifix(xls) 
      endif
      xcp = xcp(1:lenb(xcp))//'\0'
      yy = refh - tsz
      xx = refl + bw
      angle = 0.
      call ps_text (xx,yy,20,xcp,angle,10,0)
      xx = refl
      call ps_text (xx,yy,20,'0\0',angle,10,0)
      xx = refl - tsz
      yy = refh + bh 
      if (yls .lt. 0.001) then
         write (ycp,'(f6.4)') yls
      elseif (yls .lt. 0.01) then
         write (ycp,'(f5.3)') yls
      elseif (yls .lt. 0.1) then
         write (ycp,'(f4.2)') yls
      elseif (yls .lt. 1. .and. yls .ge. 0.1) then
         write (ycp,'(f3.1)') yls
      elseif (yls .lt. 100) then
         write (ycp,'(i2)') ifix(yls)
      elseif (yls .lt. 1000) then
         write (ycp,'(i3)') ifix(yls) 
      elseif (yls .lt. 10000) then
         write (ycp,'(i4)') ifix(yls)
      else
         write (ycp,'(i5)') ifix(yls)
      endif
      ycp = ycp(1:lenb(ycp))//'\0'
      call ps_text (xx,yy,20,ycp,angle,7,0)
      yy = refh
      call ps_text (xx,yy,20,'0\0',angle,7,0)

c plot semi-variogram
      do i = 1,nf
         if (i .eq. 1) then
            xfac = 1.
         else
            xfac = rescale
         endif
         do j=1,nlag(i)
            rx(j) = xlag(i,j)*scalel*xfac + refl
            ry(j) = rcor(i,j)*scaleh + refh
         enddo
         if (i .eq. 1) then
            call ps_setpaint(0,0,0)
         else
            call ps_setpaint(255,0,0)
         endif
         call ps_line(rx,ry,nlag(i),3,0)
      enddo
      call ps_setpaint(0,0,0)

c plot power law model if desired
c      if (mpowerif) then
c         call ps_setdash ('40 10 10 10',0)
c         mx(1) = 0.
c         my(1) = poffset
c         do i=2,mcorm
c            r = (i-1)*xintm
c            if (r .eq. 0.0) then
c               my(i) = poffset
c            else
c               my(i) = poffset + xcons*r**(xpon)
c            endif
c            mx(i) = xintm*(i-1)
c         enddo
c
c         do i=1,mcorm
c            mx(i) = mx(i)*scalel + refl
c            my(i) = my(i)*scaleh + refh
c         enddo
c         call ps_line(mx,my,mcorm,3,0)
c         call ps_setdash ('\0',0)
c      endif

c plot gaussian model if desired
c      if (gmodif) then
c        call ps_setdash ('40 20',0)
c         mx(1) = 0.
c         my(1) = goffset
c         do i=2,mcorm
c            r = (i-1)*xintm
c            if (r .eq. 0.0) then
c               my(i) = goffset
c            else
c               my(i) = goffset + h2g*(1.-exp(-pi*(kg*r)**2))
c            endif
c            mx(i) = xintm*(i-1)
c         enddo
c
c         do i=1,mcorm
c            mx(i) = mx(i)*scalel + refl
c            my(i) = my(i)*scaleh + refh
c         enddo
c         call ps_line(mx,my,mcorm,3,0)
c         call ps_setdash ('\0',0)
c      endif

c if desired, plot von Karman model 
      if (modelif) then
         do k = 1,nf
            call ps_setdash ('20 20',0)
            xg = g0(nu(k))
            mx(1) = 0.
            my(1) = offset(k)
            do i=2,mcorm
               r = (i-1)*xintm*k0(k)
               if (r .eq. 0.0) then
                  my(i) = offset(k)
               else
                  my(i) = offset(k) + h2(k)*
     #                    (1. - (r**nu(k))*mbsl(nu(k),r)/xg)
               endif
               mx(i) = xintm*(i-1)
            enddo
            do i=1,mcorm
               mx(i) = mx(i)*scalel + refl
               my(i) = my(i)*scaleh + refh
            enddo
            call ps_line(mx,my,mcorm,3,0)
         enddo
      endif 
      call ps_plotend (1)
      
      write (*,*) 'Please wait while plot is generated in ghostscript.'
      write (*,*) 'Close window after viewing and press <return>'
      write (*,*) 'to continue (ignore "broken pipe" message)'
      n = system('gs varplot.ps')

c option to rescale second plot if present
      rescaleif = .false.
      rescale = 1.
      if (nf .eq. 2) then 
         write (*,*) 'would you like to rescale second variogram?'
         read (*,'(a)') choice
         if (choice .eq. 'y') then
            write (*,*) 'input rescaling factor'
            read (*,*) rescale
            rescaleif = .true.
         endif
      endif
       
c option to overlay model onto plot
      modelif = .false.
      mpowerif = .false.
      gmodif = .false.
      if (a1 .eq. a2) then
         write (*,*) 'would you like to overlay von Karman model?'
         read (*,'(a)') choice
         if (choice .eq. 'y') then  
            do k = 1,nf                       
               write (*,*) 'input model ',k,' params '//
     #                     'offset, h2, k0, nu:'
               read (*,*) offset(k), h2(k), k0(k), nu(k)
            enddo
            modelif = .true.
         endif

c         write (*,*) 'would you like to overlay power law model?'
c         read (*,'(a)') choice
c         if (choice .eq. 'y') then
c            write (*,*) 'input parameters offset, h2, k0, nu:'
c            read (*,*) poffset, ph2, pk0, pnu
c            xpon = 2*pnu
c            xcons = ph2*((pk0/2.)**xpon)*gamma(1-nu)/gamma(1+nu)
c            write (*,*) 'constant = ',xcons
c            mpowerif = .true. 
c         endif

c         write (*,*) 'would you like to overlay Gaussian model?'
c         read (*,'(a)') choice
c         if (choice .eq. 'y') then
c            write (*,*) 'input parameters offset, h2g, kg:'
c            read (*,*) goffset,h2g,kg
c            gmodif = .true. 
c         endif
      endif 

      if (modelif .or. mpowerif .or. gmodif .or. rescaleif) goto 100
      
c option to try again with new scaling parameter
      write (*,*) 'try new scaling parameter?'
      read (*,'(a)') choice
      if (choice .eq. 'y') then
         goto 10
      endif  
           
999   continue
      end
      
