
c ******************************************************************
c Program written by Dr John GOFF, University of Texas at Austin

c "http://www.ig.utexas.edu/people/staff/goff/"

c Work supported by ONR Awards, 2006-7
c ******************************************************************

      program svinv

c program to invert semi-variograms calculated histogram method.  
c The von Karman covariance model
c is employed.  This inversion assumes essentially uncorrelated 
c covaraiance estimates.  Covariance function is computed in a 
c different application - this program reads in an ascii file
c with columns: (1) lag bin, (2) covariance, (3) histogram bin count
c
c Inversion Parameters and Derivatives:
c H = rms height
c H2 = variance (= H**2)
c k0 = scale parameter
c l0 = characteristic nipgth
c nu = modified bessel order parameter = Hurst number
c D = fractal dimension
c
c Author's note: Inversion for nu is highly non-linear (imagine
c trying to calculate the derivative of the modified bessel function
c with respect to it's order parameter).  The uncertainties on this 
c parameter output from this inversion tend to be significantly 
c overestimated, especially when using only one or two profiles in the 
c inversion.  For more detailed information read Goff and Jordan 
c (JGR, 1988) and Goff and Jordan (IEEE Ocean Eng., 1989)
c
c 'spec_par.inc' defines parameters,
c ncm (maximum number of points in a covariance function), 
              
      parameter (ncm=1000)
      real v(ncm,ncm),vi(ncm,ncm)
      real h,k0,data(ncm),gm0(ncm)
      real dmg(ncm),g(ncm,4),gt(4,ncm) 
      real gg(ncm,4),dm(5),vms(4)
      real gtw(4,ncm),gtwg(4,4),gtwgt(4,4),gtwgm1(4,4) 
      real ginv(4,ncm),git(ncm,4),giw(4,ncm)
      real vm(4,4),vh,vk,rs
      real mbsl,g0,g0i,nu,dkdnu,dg0dnu,rnu,vnu 
      real gmcon,gcon(4),pi
      real psi,mo(4),svar(ncm),xlag(ncm)
      real avenbin,rbin,dvar
      integer i,j,k,ninv,iinv,nbin(ncm)
      integer fp(4),nip,indx(ncm)
      logical covif,modelif,gaussif
      character infile*80
      character*1 choice,ch
       
      gaussif = .false.
      pi = 3.1415926 
      
      write (*,*) 'enter semi-variogram file name'
      read (*,'(a)') infile
      open (1,file=infile,status='old')
      read (1,*) mcor
      do i = 1,mcor
         read (1,*) xlag(i),svar(i),nbin(i)
      enddo
      rs = 1.
      
      xnoise = svar(1)
      write (*,*) 'Estimated noise variance: ',xnoise
c      write (*,*) 'Enter your value:'
c      read (*,*) xnoise
      
50    write (*,*) 'maximum lag, km'
      read (*,*) xlmax
      mcmax = mcor
      do i = 1,mcor
         if (xlag(i) .gt. xlmax) then
            mcmax = i-1
            goto 60
         endif
      enddo
60    write (*,*) mcmax
      
      write (*,*) 'input start lag index (index 1 = 0 lag):'
      read (*,*) nst
      nip = mcmax - nst + 1

c      write (*,*) 'subtract models prior to inversion?'
c      read (*,'(a)') ch
c      if (ch .eq. 'y') then
c         write (*,*) 'how many to subtract?'
c         read (*,*) nsub
c         do isub = 1,nsub
c            write (*,*) 'enter parameters H2,k0,nu:'
c            read (*,*) xsh,xsk0,xsnu
c            svar(1) = svar(1) - xsh
c            do i = 2,mcor
c               r = xlag(i)*xsk0
c               xc = xsh*(r**xsnu)*mbsl(xsnu,r)/g0(xsnu)
c               svar(i) = svar(i) - xc
c            enddo
c         enddo
c      endif
      
c set up data array 
c use nip points of the autocovariance starting
c with nst'th point 
      
      do j = 1,nip
         data(j) = svar(j+nst-1)/rs
      enddo           

      write (*,*) 'preview semi-variogram function?'
      read (*,'(a)') ch
      if (ch .eq. 'y') then
         modelif = .false.
         call varplot(svar,mcor,xlag,0.,1.,1.,0.5,modelif,gaussif)
      endif
      write (*,*) 'start again with new sample parameters? '
      read (*,'(a)') ch
      if (ch .eq. 'y') goto 50

c start inversion
c input starting parameters 
      write (*,*) 'input starting parameters: N2,H2,Ko,nu'
      read (*,*) xn,h,k0,nu 
      mo(1) = xn
      mo(2) = h
      mo(3) = k0
      mo(4) = nu
      write (*,*) 'enter free parameter codes for each parameter'
      write (*,*) '0 = fixed, free otherwise'
      read(*,*) fp(1),fp(2),fp(3),fp(4)
      write (*,*) 'include covariance?'
      read (*,'(a)') ch
      if (ch .eq. 'y') then
         covif = .true.
      else
         covif = .false.
      endif

      write (*,*) 'input number of inversions'
      read (*,*) ninv
      nfp = 0
      do i = 1,4
         if (fp(i) .ne. 0) then
            fp(i) = 1
         endif
         nfp = nfp + fp(i)
      enddo
   
      goto 120

100   continue
      write (*,*) 'do another inversion? (y or n)'
      read (*,'(a)') choice
      if (choice .ne. 'y') then
         write (*,*) 'start again with new parameters?'
         read (*,'(a)') ch
         if (ch .ne. 'y') then
            goto 999
         else 
            write (*,*) 'input starting parameters: N2,H2,Ko,nu'
            read (*,*) xn,h,k0,nu
            mo(1) = xn
            mo(2) = h
            mo(3) = k0
            mo(4) = nu
            write (*,*) 'enter free parameter codes for each parameter'
            write (*,*) '0 = fixed, free otherwise'
            read(*,*) fp(1),fp(2),fp(3),fp(4)
            write (*,*) 'include covariance?'
            read (*,'(a)') ch
            if (ch .eq. 'y') then
               covif = .true.
            else
               covif = .false.
            endif
         endif
      endif
      write (*,*) 'input number of inversions'
      read (*,*) ninv 
      nfp = 0
      do i = 1,4
         if (fp(i) .ne. 0) then
            fp(i) = 1
         endif
         nfp = nfp + fp(i)
      enddo

120   continue

      do iinv = 1,ninv 
      if (nu .le. 0. .or. nu .ge. 1.) goto 100
c determine starting model gm0  
      g0i = 1/g0(nu)
      do i = 1,nip
         gmcon = 0.
         r = k0*xlag(i+nst-1)
         if (r .eq. 0) then
            gm0(i) = xn
         else
            gm0(i) = xn + h*(1.0 - (r**nu)*
     #               mbsl(nu,r)*g0i)
         endif
      enddo
                                                 
c find difference between starting model and data
      dvar = 0.
      do i = 1,nip
         dmg(i) = (data(i) - gm0(i))
         dvar = dvar + dmg(i)**2
      enddo
      dvar = dvar/float(nip)
      
      avenbin = 0.
      do i = 1,nip
         avenbin = avenbin + float(nbin(i+nst-1))
      enddo
      avenbin = avenbin/float(nip)
      write (*,*) dvar, avenbin

c determine sensitivity matrix (g) and its transform (gt) 
      xpsi = psi(nu)
      dg0dnu = (g0(nu+.0005) - g0(nu-.0005))/.001
      do i = 1,nip 
         do j = 1,4
            gcon(j) = 0.
         enddo
         x = xlag(i+nst-1)
         r = x*k0
         if (r .eq. 0) then
            gg(i,1) = 1.
            gg(i,2) = 1.
            gg(i,3) = 0.
            gg(i,4) = 0.
         else
            dkdnu = (mbsl((nu+.005),r) - mbsl((nu-.005),r))/.01
            rnu = r**nu
            gg(i,1) = 1.
            gg(i,2) = 1. - rnu*mbsl(nu,r)*g0i
            gg(i,3) = h*rnu*x*mbsl((1.-nu),r)*g0i
            gg(i,4) = -h*rnu*(mbsl(nu,r)*g0i*
     #               (log(r/2.)-xpsi) + dkdnu)
         endif
      enddo    
c compress matrix
      do i = 1,nip 
         k = 0
         do j = 1,4
            k = k + fp(j)
            if (fp(j) .ne. 0) then
               g(i,k) = gg(i,j)      
            endif
         enddo
      enddo   
               
c transform of sensitivity matrix   
      do i = 1,nfp
         do j = 1,nip
            gt(i,j) = g(j,i)
         enddo
      enddo

c calculate covariance matrix v and its inverse vi
      if (covif) then
         do i = 1,nip
            do j = 1,nip
               v(i,j) = 0.
               vi(i,j) = 0.
            enddo
            rbin = float(nbin(i+nst-1))/avenbin
c            v(i,i) = ((xn+h)/nbin(i+nst-1))**2
            v(i,i) = dvar/rbin
            vi(i,i) = 1/v(i,i)
         enddo
      else
         do i = 1,nip
            do j = 1,nip
               v(i,j) = 0.
               vi(i,j) = 0.
            enddo
            v(i,i) = 1.
            vi(i,i) = 1.
         enddo
      endif

c do matrix operations for inversion
c define GtransW (gtw)
      do i = 1,nfp
         do j = 1,nip
         gtw(i,j) = 0.
            do k = 1,nip
               gtw(i,j) = gt(i,k)*vi(k,j) + gtw(i,j)
            enddo
         enddo
      enddo

c define GtransWG (gtwg)
      do i=1,nfp
         do j = 1,nfp
            gtwg(i,j) = 0.
            do k = 1,nip
               gtwg(i,j) = gtwg(i,j) + gtw(i,k)*g(k,j)
            enddo
         enddo
      enddo

c find inverse of gtwg = gtwgm1
      do i = 1,4
         do j = 1,4
            gtwgt(i,j) = gtwg(i,j)
         enddo
      enddo
      call lu_matinv(gtwgt,gtwgm1,nfp,4,indx)
                                      
c find inverse operator ginv 
      do i = 1,nfp
         do j = 1,nip
            ginv(i,j) = 0.
            do k = 1,nfp
               ginv(i,j) = gtwgm1(i,k)*gtw(k,j) + ginv(i,j)
            enddo
         enddo
      enddo
         
c solve for updated parameters
      do i = 1,nfp
         dm(i) = 0
         do j = 1,nip
            dm(i) = ginv(i,j)*dmg(j) + dm(i)
         enddo
      enddo
      j = 0
      do i = 1,4
         j = j + fp(i)
         mo(i) = mo(i) + float(fp(i))*dm(j)
      enddo
      xn = mo(1)
      h = mo(2)
      k0 = mo(3)
      nu = mo(4)

c find error on estimates of parameters
c find Ginvtrans (git) 
      do i = 1,nfp
         do j = 1,nip
            git(j,i) = ginv(i,j)
         enddo
      enddo
      
c determine Ginv*W  (giw)
      do i = 1,nfp
         do j = 1,nip
            giw(i,j) = 0.
            do k = 1,nip
               giw(i,j) = ginv(i,k)*v(k,j) + giw(i,j)
            enddo
         enddo
      enddo
     
c determine model covariance
      do i = 1,nfp
         do j = 1,nfp
            vm(i,j) = 0.
            do k = 1,nip
               vm(i,j) = giw(i,k)*git(k,j) + vm(i,j)
            enddo                                   
         enddo
      enddo             
      j = 0
      do i = 1,4
         j = j + fp(i)
         vms(i) = float(fp(i))*sqrt(abs(vm(j,j)))
         if (vm(j,j) .lt. 0. .and. fp(i) .eq. 1) then
            write (*,*) 'warning: vm(',j,',',j,') < 0'
         endif
      enddo
      vh = sqrt(abs(vm(1,1)))
      vk = sqrt(abs(vm(2,2)))
      vnu = sqrt(abs(vm(3,3)))
      if (vnu .ne. 0. .and. vk .ne. 0.) then
         rho23 = vm(2,3)/vk*vnu
      else
         rho23 = 0.
      endif
      rho23 = 0.8
      write (*,*) 'N2 = ',xn,' +- ',vms(1)
      write (*,*) 'H2 = ',h,' +- ',vms(2)
      write (*,*) '(H = ',sqrt(h),' +-',vms(2)/(2.*sqrt(h)),')'
      write (*,*) 'ko = ',k0,' +- ',vms(3)                               
      xl0 = 2.*sqrt(2.*(nu+0.5))/k0
      vxl0 = sqrt((2./k0**2)*(vms(4)**2 + 4*(nu+0.5)*vms(3)**2/k0**2 -
     #       2.*nu*sqrt(nu+0.5)*vms(3)*vms(4)*rho23/(nu*k0)))
      write (*,*) '(lo = ',xl0,' +- ',vxl0,')'
      write (*,*) 'nu = ',nu,' +- ',vms(4)
      write (*,*) ' '
      
      enddo
      goto 100

999   continue
      write (*,*) 'plot inversion results?'
      read (*,'(a)') ch
      if (ch .eq. 'y') then
         modelif = .true.
         call varplot(svar,mcor,xlag,xn,h,k0,nu,
     #                modelif,gaussif)
      endif
      
      write (*,*) 'start again?'
      read (*,'(a)') ch
      if (ch .eq. 'y') goto 50

      end
 
